<?php

namespace Drupal\layout_builder_animations\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation of the 'baumeister_animation_field_type' field type.
 *
 * @FieldType(
 *   id = "layout_builder_animations_field_type",
 *   label = @Translation("Animation"),
 *   description = @Translation("Animation"),
 *   default_widget = "layout_builder_animations_widget",
 *   default_formatter = "layout_builder_animations_formatter"
 * )
 */
class AnimationFieldType extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [] + parent::defaultStorageSettings();
  }

  /**
   * Provide a list of animations.
   *
   * @return array
   *   Animation types.
   */
  public static function getAnimations() {
    return [
      '' => t('None'),
      (string) t('Attention Seekers') => [
        'bounce' => t('Bounce'),
        'flash' => t('Flash'),
        'pulse' => t('Pulse'),
        'rubberBand' => t('Rubberband'),
        'shakeX' => t('Shake X'),
        'shakeY' => t('Shake Y'),
        'headShake' => t('Headshake'),
        'swing' => t('Swing'),
        'tada' => t('Tada'),
        'wobble' => t('Wobble'),
        'jello' => t('Jello'),
        'heartBeat' => t('Heartbeat'),
      ],
      (string) t('Back entrances') => [
        'backInDown' => t('Back in down'),
        'backInLeft' => t('Back in left'),
        'backInRight' => t('Back in right'),
        'backInUp' => t('Back in up'),
      ],
      (string) t('Bouncing entrances') => [
        'bounceIn' => t('Bounce in'),
        'bounceInDown' => t('Bounce in down'),
        'bounceInLeft' => t('Bounce in left'),
        'bounceInRight' => t('Bounce in right'),
        'bounceInUp' => t('Bounce in up'),
      ],
      (string) t('Fading entrances') => [
        'fadeIn' => t('Bounce in'),
        'fadeInDown' => t('Bounce in down'),
        'fadeInDownBig' => t('Bounce in left'),
        'fadeInLeft' => t('Bounce in down'),
        'fadeInLeftBig' => t('Bounce in left'),
        'fadeInRight' => t('Bounce in down'),
        'fadeInRightBig' => t('Bounce in left'),
        'fadeInUp' => t('Bounce in down'),
        'fadeInUpBig' => t('Bounce in left'),
        'fadeInTopLeft' => t('Bounce in down'),
        'fadeInTopRight' => t('Bounce in left'),
        'fadeInBottomLeft' => t('Bounce in down'),
        'fadeInBottomRight' => t('Bounce in left'),
      ],
      (string) t('Rotating entrances') => [
        'rotateIn' => t('Bounce in'),
        'rotateInDownLeft' => t('Bounce in down'),
        'rotateInDownRight' => t('Bounce in down'),
        'rotateInUpLeft' => t('Bounce in down'),
        'rotateInUpRight' => t('Bounce in down'),
      ],
      (string) t('Zooming entrances') => [
        'zoomIn' => t('Bounce in'),
        'zoomInDown' => t('Bounce in down'),
        'zoomInLeft' => t('Bounce in down'),
        'zoomInRight' => t('Bounce in down'),
        'zoomInUp' => t('Bounce in down'),
      ],
      (string) t('Sliding entrances') => [
        'slideInDown' => t('Bounce in'),
        'slideInLeft' => t('Bounce in down'),
        'slideInRight' => t('Bounce in down'),
        'slideInUp' => t('Bounce in down'),
      ],
      (string) t('Flippers') => [
        'flip' => t('Bounce in'),
        'flipInX' => t('Bounce in down'),
        'flipInY' => t('Bounce in left'),
      ],
      (string) t('Lightspeed') => [
        'lightSpeedInRight' => t('Bounce in'),
        'lightSpeedInLeft' => t('Bounce in down'),
      ],
      (string) t('Specials') => [
        'jackInTheBox' => t('Bounce in'),
        'rollIn' => t('Bounce in down'),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function mainPropertyName() {
    return 'animation';
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    // Treat the values as property value of the main property, if no array is
    // given.
    if (isset($values) && !is_array($values)) {
      $values = [static::mainPropertyName() => $values];
    }
    parent::setValue($values, $notify);
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    // Prevent early t() calls by using the TranslatableMarkup.
    $properties['animation'] = DataDefinition::create('string');
    $properties['duration'] = DataDefinition::create('integer');
    $properties['delay'] = DataDefinition::create('integer');

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $schema = [
      'columns' => [
        'animation' => [
          'type' => 'varchar',
          'length' => 64,
        ],
        'duration' => [
          'type' => 'int',
        ],
        'delay' => [
          'type' => 'int',
        ],
      ],
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->get('animation')->getValue();
    return $value === NULL || $value === '';
  }

}
